package io.gitlab.jfronny.googlechat.mixin;

import io.gitlab.jfronny.googlechat.GoogleChat;
import io.gitlab.jfronny.googlechat.GoogleChatConfig;
import io.gitlab.jfronny.libjf.translate.Language;
import io.gitlab.jfronny.libjf.translate.TranslateException;
import io.gitlab.jfronny.libjf.translate.TranslateService;
import net.fabricmc.api.EnvType;
import net.fabricmc.loader.api.FabricLoader;
import net.minecraft.network.MessageType;
import net.minecraft.network.PacketByteBuf;
import net.minecraft.network.packet.s2c.play.GameMessageS2CPacket;
import net.minecraft.text.LiteralText;
import net.minecraft.text.Text;
import net.minecraft.util.Util;
import org.spongepowered.asm.mixin.Final;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Mutable;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

import java.util.UUID;

@Mixin(GameMessageS2CPacket.class)
public class GameMessageS2CPacketMixin {
    @Shadow @Final @Mutable private Text message;
    @Shadow @Final private UUID sender;

    @Inject(at = @At("RETURN"), method = "<init>(Lnet/minecraft/text/Text;Lnet/minecraft/network/MessageType;Ljava/util/UUID;)V")
    private void init(Text message, MessageType type, UUID sender, CallbackInfo ci) {
        googlechat$translate();
    }

    @Inject(at = @At("RETURN"), method = "<init>(Lnet/minecraft/network/PacketByteBuf;)V")
    private void init(PacketByteBuf buf, CallbackInfo ci) {
        googlechat$translate();
    }

    private void googlechat$translate() {
        if (!GoogleChatConfig.enabled) return;
        if (FabricLoader.getInstance().getEnvironmentType() == EnvType.CLIENT && GoogleChat.isSelf(sender)) {
            return;
        }
        Language client = Language.byId(GoogleChatConfig.clientLanguage);
        if (client == Language.AUTO_DETECT) return;
        StringBuilder sb = new StringBuilder();
        message.asOrderedText().accept((index, style, codePoint) -> {
            sb.append((char)codePoint);
            return true;
        });
        try {
            message = new LiteralText(TranslateService.translate(sb.toString(), Language.byId(GoogleChatConfig.serverLanguage), client));
        } catch (TranslateException e) {
            GoogleChat.LOGGER.error("Could not translate received message", e);
        }
    }
}
